﻿using System;

using Android.App;
using Android.Content;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;

using System.Collections.Generic;
using Android.Util;
using MyClass;


namespace IrTimer
{
    /// <summary>
    /// Cusotm list adapter.
    /// </summary>
    public class CusotmListAdapter : BaseAdapter<MySetTime.TimerExecInfo>
	{
        #region 定数
        private const string TAG = "CusotmListAdapter";


        /// <summary>ポップアップメニューID:削除</summary>
        private const int POPUP_MENU_ID_DEL         = 0;
        /// <summary>ポップアップメニューID:項目を1つ上に</summary>
        private const int POPUP_MENU_ID_ITEM_UP     = 1;
        /// <summary>ポップアップメニューID:項目を1つ下に</summary>
        private const int POPUP_MENU_ID_ITEM_DOWN   = 2;
        #endregion //定数


        #region フィールド
		/// <summary></summary>
        private Activity context;
		/// <summary></summary>
        private List<MySetTime.TimerExecInfo> list;
        /// <summary></summary>
        private MySetTime.RemoconItem[] remoconItem;
        #endregion //フィールド


        #region プロパティ
        #region Count
        /// <summary>Count</summary>
        /// <value>To be added.</value>
        public override int Count
        {
            get
            {
                return list.Count;
            }
        }
        #endregion //Count


        #region GetItemId
        /// <summary>GetItemId</summary>
        /// <value>To be added.</value>
        public override long GetItemId (int position)
        {
            return position;
        }
        #endregion //GetItemId


        #region MySetTime.TimerExecInfo
        /// <summary>MySetTime.TimerExecInfo</summary>
        /// <value>To be added.</value>
        public override MySetTime.TimerExecInfo this[int index]
        {
            get 
            {
                return list [index]; 
            }
        }
        #endregion //MySetTime.TimerExecInfo
        #endregion //プロパティ


        #region コンストラクタ
		/// <summary>コンストラクタ</summary>
		/// <param name="_context">Context.</param>
		/// <param name="_list">List.</param>
        public CusotmListAdapter (Activity _context, List<MySetTime.TimerExecInfo> _list, MySetTime.RemoconItem[] _remoconItem) :base()
		{
			this.context = _context;
			this.list = _list;
            this.remoconItem = _remoconItem;
		}
        #endregion //コンストラクタ


        #region GetView
		/// <param name="position">The position of the item within the adapter's data set of the item whose view we want.</param>
		/// <summary>Gets the view.</summary>
		/// <returns>The view.</returns>
		/// <param name="convertView">Convert view.</param>
		/// <param name="parent">Parent.</param>
		public override View GetView (int position, View convertView, ViewGroup parent)
		{
			View view = convertView; 

            Log.WriteLine(LogPriority.Info, TAG, string.Format("GetView, pos={0}", position));
			// re-use an existing view, if one is available otherwise create a new one
			//if (view == null)
			{
                // UIの反映
                view = context.LayoutInflater.Inflate(Resource.Layout.row, parent, false);

                TextView textRow = view.FindViewById<TextView>(Resource.Id.textViewRow);

                textRow.Text = getTimeText(list[position]);

                #region 有効/無効チェックボックス
                CheckBox tb = view.FindViewById<CheckBox>(Resource.Id.checkBoxEnable);

                tb.Checked = list[position].IsEnable;
                tb.CheckedChange += delegate
                    {
                        var item = new MySetTime (context);
                        item.RemoconItems = remoconItem;
                        item.TimerInfo = list[position];

                        MyClass.MySetTime.TimerExecInfo ti = item.TimerInfo;
                        ti.IsEnable = tb.Checked;

                        list[position] = ti;
                    };
                #endregion //有効/無効チェックボックス

                view.Click += delegate
                    {
                        var item = new MySetTime (context);
                        item.RemoconItems = remoconItem;
                        item.TimerInfo = list[position];

                        // アラートダイアログ を生成
                        AlertDialog.Builder builder = new AlertDialog.Builder(context);
                        builder.SetTitle("設定");
                        builder.SetView(item);
                        builder.SetPositiveButton("OK", delegate
                            {
                                // OK ボタンクリック処理
                                MyClass.MySetTime.TimerExecInfo ti = item.TimerInfo;

                                textRow.Text = getTimeText(ti);

                                list[position] = ti;

                                context.FindViewById<ListView> (Resource.Id.listViewTimeSettings).Adapter = new CusotmListAdapter(context, list, remoconItem);//追加すると改善するが…要検討
                            });

                        builder.SetNegativeButton("キャンセル", delegate
                            {
                                // Cancel ボタンクリック処理
                            });

                        // 表示
                        builder.Create().Show();
                    };

                #region オプションボタン
                ImageButton buttonOption = view.FindViewById<ImageButton>(Resource.Id.imageButtonOption);
                buttonOption.Click += delegate
                    {
                        PopupMenu popup = new PopupMenu(context, buttonOption);
                        popup.Menu.Add(Menu.None, POPUP_MENU_ID_DEL, Menu.None, "削除");
                        popup.Menu.GetItem(POPUP_MENU_ID_DEL).SetIcon(Android.Resource.Drawable.IcDelete);

                        popup.Menu.Add(Menu.None, POPUP_MENU_ID_ITEM_UP, Menu.None, "1つ上に移動");
                        popup.Menu.GetItem(POPUP_MENU_ID_ITEM_UP).SetIcon(Android.Resource.Drawable.IcMenuUpload);

                        popup.Menu.Add(Menu.None, POPUP_MENU_ID_ITEM_DOWN, Menu.None, "1つ下に移動");
                        popup.Menu.GetItem(POPUP_MENU_ID_ITEM_DOWN).SetIcon(Android.Resource.Drawable.ArrowDownFloat);

                        popup.MenuItemClick += (sender, e) => 
                            {
                                switch(e.Item.ItemId)
                                {
                                    case POPUP_MENU_ID_DEL:
                                        list.RemoveAt(position);
                                        context.FindViewById<ListView> (Resource.Id.listViewTimeSettings).Adapter = new CusotmListAdapter(context, list, remoconItem);
                                        break;

                                    case POPUP_MENU_ID_ITEM_UP:
                                        if(position > 0)
                                        {
                                            MySetTime.TimerExecInfo tmp = list[position];
                                            list[position] = list[position-1];
                                            list[position-1] = tmp;
                                            context.FindViewById<ListView> (Resource.Id.listViewTimeSettings).Adapter = new CusotmListAdapter(context, list, remoconItem);
                                        }
                                        break;

                                    case POPUP_MENU_ID_ITEM_DOWN:
                                        if(position < (list.Count-1))
                                        {
                                            MySetTime.TimerExecInfo tmp = list[position];
                                            list[position] = list[position+1];
                                            list[position+1] = tmp;
                                            context.FindViewById<ListView> (Resource.Id.listViewTimeSettings).Adapter = new CusotmListAdapter(context, list, remoconItem);
                                        }
                                        break;
                                }
                            };
                        popup.Show();

                    };
                #endregion //オプションボタン
			}

			return view;
		}
        #endregion //GetView


        #region 時刻情報を表示形式に整形する
        /// <summary>時刻情報を表示形式に整形する</summary>
        /// <returns>The text.</returns>
        /// <param name="ti">Ti.</param>
        private string getTimeText(MyClass.MySetTime.TimerExecInfo ti)
        {
            string text;

            if (ti.TimerKind == MySetTime.TimerKinds.CountDown)
            {
                text = "カウントダウン ";
                text += string.Format("({0:D}:{1:D2})", ti.CountDownTimerHour, ti.CountDownTimerMinute);
            }
            else
            {
                text = "時刻指定 ";

                string ap;
                if (ti.TimeHour >= 12)
                {
                    ap = "午後";
                }
                else
                {
                    ap = "午前";
                }

                int h;
                if ((ti.TimeHour == 0) || (ti.TimeHour == 12)) 
                {
                    h = 12;
                }
                else if(ti.TimeHour < 12)
                {
                    h = ti.TimeHour;
                }
                else
                {
                    h = ti.TimeHour - 12;
                }

                text += string.Format("({2} {0:D}:{1:D2})", h, ti.TimeMinute, ap);      // 24H表記の場合
            }

            text += string.Format(" ({0} / {1})", ti.RemoconGroup, ti.RemoconCmd);

            return text;
        }
        #endregion //時刻情報を表示形式に整形する



	}
}


