/*
* Copyright (C) 2009 The Android Open Source Project
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

using System;
using Android.App;
using Android.Bluetooth;
using Android.Content;
using Android.OS;
using Android.Util;
using Android.Views;
using Android.Views.InputMethods;
using Android.Widget;
using Java.Lang;

using Android.Preferences;

using System.Collections.Generic;
using MyClass;







namespace IrTimer
{
	/// <summary>
	/// This is the main Activity that displays the current chat session.
	/// </summary>
    [Activity  (Label = "@string/app_name", MainLauncher = true, 
                Theme = "@android:style/Theme.Holo",
	            ConfigurationChanges=Android.Content.PM.ConfigChanges.KeyboardHidden | Android.Content.PM.ConfigChanges.Orientation)]
    public class MainActivity : Activity
	{
        #region 定数
		// Debugging
		private const string TAG = "BluetoothChat";
		private const bool Debug = true;
	
        // 設定ファイル系
        private const string SETTING_COMPLETE_AFTER_SHUTDOWN_KIND   = "CompleteAfterShutdownKind";

        private const string SETTING_COUNT              = "Count";
        private const string SETTING_ENABLE             = "Enable";
        private const string SETTING_TIMER_KIND         = "TimerKind";

        private const string SETTING_COUNT_DOWN_HOUR    = "CountDownHour";
        private const string SETTING_COUNT_DOWN_MINUTE  = "CountDownMinute";
        private const string SETTING_TIME_HOUR          = "TimeHour";
        private const string SETTING_TIME_MINUTE        = "TimeMinute";

        private const string SETTING_REMOCON_GROUP      = "RemoconGroup";
        private const string SETTING_REMOCON_CMD        = "RemoconCmd";

        // リモコングループ/コマンド設定ファイル系
        private const string REMOCON_FILENAME           = "remoconGroup";

        private const string REMOCON_GROUP_COUNT        = "GroupCount";
        private const string REMOCON_GROUP_NAME         = "GroupName";
        private const string REMOCON_COMMAND_COUNT      = "CommandCount";
        private const string REMOCON_COMMAND_NAME       = "CommandName";



		// Key names received from the BluetoothChatService Handler
		public const string DEVICE_NAME = "device_name";
		public const string TOAST = "toast";

		// Intent request codes
		// TODO: Make into Enums
		private const int REQUEST_CONNECT_DEVICE = 1;
		private const int REQUEST_ENABLE_BT = 2;


        #region コマンドID
        /// <summary>コマンドID : 実行ボタン押下</summary>
        private const ushort CMD_ID_ACCEPT      = 0x1001;
        /// <summary>コマンドID : キャンセルボタン押下</summary>
        private const ushort CMD_ID_CANCEL      = 0x1002;
        /// <summary>コマンドID : 新フォーマット方式</summary>
        private const ushort CMD_ID_NEW_FORMAT  = 0x1000;
        /// <summary>コマンドID : コマンド名取得ボタン</summary>
        private const ushort CMD_ID_GET_CMD_NAME= 0x1003;

        /// <summary>コマンドID : レスポンス</summary>
        private const ushort CMD_ID_RESPONSE    = 0x2000;
        #endregion //コマンドID


        #region レスポンス
        /// <summary>レスポンスID</summary>
        private enum RESPONSE_ID : byte
        {
            /// <summary>レスポンス : 正常終了</summary>
            Success             = 0,
            /// <summary>レスポンス : 異常終了</summary>
            Failure             = 1,
            /// <summary>レスポンス : 実行中のため失敗</summary>
            FailureForRunning   = 2,
            /// <summary>レスポンス : 設定数が0のため失敗</summary>
            FailureSettingZero  = 3,
        }
        #endregion //レスポンス





        /// <summary></summary>
        private const string CDT_NAME = "RasPi-IrTimer";

        /// <summary>設定数の上限</summary>
        private const int SETTING_COUNT_MAX   = 10;

        /// <summary></summary>
        private const int CMD_LENGTH          = 256;

        #endregion //定数


        #region フィールド
		// Layout Views
        /// <summary></summary>
        protected TextView textViewTitle;
	
        /// <summary>Name of the connected device</summary>
		protected string connectedDeviceName = null;
        /// <summary>Local Bluetooth adapter</summary>
		private BluetoothAdapter bluetoothAdapter = null;
        /// <summary>Member object for the chat services</summary>
        private BluetoothService btComService = null;

        /// <summary>時間設定をリストで管理</summary>
        List<MySetTime.TimerExecInfo> settingList = new List<MySetTime.TimerExecInfo> ();
        /// <summary>設定:終了後の動作</summary>
        private MySetting.COMPLETE_AFTER_SHUTDOWN_KIND settingCompleteAfterShutdown = MySetting.COMPLETE_AFTER_SHUTDOWN_KIND.KEEP;

        /// <summary>当アプリでBluetoothを有効にしたらtrue</summary>
        private bool isBluetoothEnable = false;

        /// <summary></summary>
        private TextView textViewStatus;
        /// <summary></summary>
        private Button buttonSetting;
        /// <summary></summary>
        private Button buttonAccept;
        /// <summary></summary>
        private Button buttonCancel;
        /// <summary></summary>
        //private Button buttonGetCmdName;

        /// <summary></summary>
        //private Button buttonDebug;

        /// <summary>リモコングループ/コマンド情報</summary>
        MySetTime.RemoconItem[] remoconItem;
        //MySetTime.RemoconItem[] remoconItem = new MySetTime.RemoconItem[0];
        //MySetTime.RemoconItem[] remoconItem = new MySetTime.RemoconItem[1]{new MySetTime.RemoconItem()};
        #endregion //フィールド



        #region コンストラクタ相当の物
        /// <summary>コンストラクタ相当の物</summary>
        /// <param name="bundle">Bundle.</param>
		protected override void OnCreate (Bundle bundle)
		{
			base.OnCreate (bundle);
			
			if (Debug) Log.Error (TAG, "+++ ON CREATE +++");
			

			// Set up the window layout
			SetContentView (Resource.Layout.main);


            textViewStatus              = FindViewById<TextView>(Resource.Id.textViewStatus);
            buttonSetting               = FindViewById<Button>(Resource.Id.buttonSetting);
            buttonAccept                = FindViewById<Button>(Resource.Id.buttonAccept);
            buttonCancel                = FindViewById<Button>(Resource.Id.buttonCancel);

            var listView = FindViewById<ListView>(Resource.Id.listViewTimeSettings);

            buttonSetting.Click      += new EventHandler(buttonSetting_Click);
            buttonAccept.Click       += new EventHandler(buttonAccept_Click);
            buttonCancel.Click       += new EventHandler(buttonCancel_Click);


            #region 設定リスト
            // 追加ボタン
            var buttonAdd = FindViewById<Button>(Resource.Id.buttonAdd);
            buttonAdd.Click += delegate
                {
                    if(settingList.Count < SETTING_COUNT_MAX)
                    {
                        // ダイアログを表示してから項目を追加する
                        var item = new MySetTime (this);
                        item.RemoconItems = remoconItem;
                        item.TimerInfo = new MySetTime.TimerExecInfo();     // 通常はクラス生成時に自動で初期化されるが、処理上の都合で別途newして代入する

                        // アラートダイアログ を生成
                        AlertDialog.Builder builder = new AlertDialog.Builder(this);
                        builder.SetTitle("設定");
                        builder.SetView(item);
                        builder.SetPositiveButton("OK", delegate
                            {
                                // OK ボタンクリック処理
                                MySetTime.TimerExecInfo ti = item.TimerInfo;

                                settingList.Add(ti);
                                listView.Adapter = new CusotmListAdapter(this, settingList, remoconItem);
                            });

                        builder.SetNegativeButton("キャンセル", delegate
                            {
                                // Cancel ボタンクリック処理
                            });

                        // 表示
                        builder.Create().Show();
                    }
                };
            #endregion //設定リスト


            #region 設定の復元
            var sp = GetPreferences(FileCreationMode.Private);

            settingCompleteAfterShutdown = (MySetting.COMPLETE_AFTER_SHUTDOWN_KIND)sp.GetInt(string.Format("{0}", SETTING_COMPLETE_AFTER_SHUTDOWN_KIND), (byte)MySetting.COMPLETE_AFTER_SHUTDOWN_KIND.KEEP);

            for(int i=0; i<sp.GetInt(SETTING_COUNT, 0); i++)
            {
                MySetTime.TimerExecInfo ti = new MySetTime.TimerExecInfo();

                ti.IsEnable = sp.GetBoolean(string.Format("{0}{1:D}", SETTING_ENABLE, i), true);

                if(sp.GetBoolean(string.Format("{0}{1:D}", SETTING_TIMER_KIND, i), true))
                {
                    ti.TimerKind = MySetTime.TimerKinds.CountDown;
                }
                else
                {
                    ti.TimerKind = MySetTime.TimerKinds.Time;
                }

                ti.CountDownTimerHour   = sp.GetInt(string.Format("{0}{1:D}", SETTING_COUNT_DOWN_HOUR, i), 1);
                ti.CountDownTimerMinute = sp.GetInt(string.Format("{0}{1:D}", SETTING_COUNT_DOWN_MINUTE, i), 0);

                ti.TimeHour             = sp.GetInt(string.Format("{0}{1:D}", SETTING_TIME_HOUR, i), 1);
                ti.TimeMinute           = sp.GetInt(string.Format("{0}{1:D}", SETTING_TIME_MINUTE, i), 0);

                ti.RemoconGroup         = sp.GetString(string.Format("{0}{1:D}", SETTING_REMOCON_GROUP, i), "");
                ti.RemoconCmd           = sp.GetString(string.Format("{0}{1:D}", SETTING_REMOCON_CMD, i), "");

                settingList.Add(ti);
            }
            #endregion //設定の復元


            #region リモコングループ/コマンド情報の復元
            var gsp = GetSharedPreferences(REMOCON_FILENAME, FileCreationMode.Private);

            int groupCount = gsp.GetInt(REMOCON_GROUP_COUNT, 0);
            if(groupCount > 0)
            {
                remoconItem = new MySetTime.RemoconItem[groupCount];

                for(int i=0; i<groupCount; i++)
                {
                    remoconItem[i].GroupName = gsp.GetString(string.Format("{0}{1:D}", REMOCON_GROUP_NAME, i), "");

                    int cmdCount = gsp.GetInt(string.Format("{0}{1:D}",REMOCON_COMMAND_COUNT, i), 0);
                    remoconItem[i].CmdName = new string[cmdCount];

                    for(int j=0; j<cmdCount; j++)
                    {
                        remoconItem[i].CmdName[j] = gsp.GetString(string.Format("{0}{1:D}-{2:D}", REMOCON_COMMAND_NAME, i, j), "");
                    }
                }

            }
            else
            {
                remoconItem = new MySetTime.RemoconItem[1]{new MySetTime.RemoconItem()};
                remoconItem[0].GroupName = "";
                remoconItem[0].CmdName = new string[]{""};
            }
            #endregion //リモコングループ/コマンド情報の復元


            // 設定の復元後にlistViewをセット
            listView.Adapter = new CusotmListAdapter(this, settingList, remoconItem);



            #region Bluetooth アダプタの取得
            // Get local Bluetooth adapter
			bluetoothAdapter = BluetoothAdapter.DefaultAdapter;
	
#if !DEBUG                          // デバッグ時は無効にする
			// If the adapter is null, then Bluetooth is not supported
            if (bluetoothAdapter == null)
            {
                //Toast.MakeText(this, "Bluetooth is not available", ToastLength.Long).Show();
                Toast.MakeText(this, Resource.String.bt_not_available, ToastLength.Long).Show();
                Finish();
                return;
            }
#endif
            #endregion //Bluetooth アダプタの取得
		}
        #endregion //コンストラクタ相当の物


        #region 「設定」ボタン
        /// <summary>「設定」ボタン</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonSetting_Click(object sender, EventArgs e)
        {
            // 有効な項目数をカウント
            byte count = 0;
            for (int i = 0; i < settingList.Count; i++)
            {
                if (settingList[i].IsEnable)
                {
                    count++;
                }
            }


            // まずコマンドICとグループ数、設定を送信
            byte[] sendData = new byte[4];
            BitConverter.GetBytes(CMD_ID_NEW_FORMAT).CopyTo(sendData, 0);   // 0～1
            sendData[2] = (byte)settingCompleteAfterShutdown;
            sendData[3] = count;

            if (SendMessage(sendData) == false)
            {
                Toast.MakeText(this, Resource.String.setting_fail, ToastLength.Short).Show();
                return;
            }


            // 続いてデータ
            int ln = 3 + CMD_LENGTH * 2;
            sendData = new byte[ln];

            for (int i = 0; i < settingList.Count; i++)
            {
                if (settingList[i].IsEnable)
                {
                    if(settingList[i].TimerKind == MySetTime.TimerKinds.CountDown)
                    {
                        sendData[0] = 0;                                            // 0:カウントダウンモード 1:タイマーモード
                        sendData[1] = (byte)settingList[i].CountDownTimerHour;      // hh
                        sendData[2] = (byte)settingList[i].CountDownTimerMinute;    // mm
                    }
                    else
                    {
                        sendData[0] = 1;                                             // 0:カウントダウンモード 1:タイマーモード
                        sendData[1] = (byte)settingList[i].TimeHour;                 // hh
                        sendData[2] = (byte)settingList[i].TimeMinute;               // mm
                    }
                    Array.Copy(System.Text.Encoding.ASCII.GetBytes(settingList[i].RemoconGroup), 0, sendData, 3,   settingList[i].RemoconGroup.Length);
                    Array.Copy(System.Text.Encoding.ASCII.GetBytes(settingList[i].RemoconCmd),   0, sendData, 259, settingList[i].RemoconCmd.Length);

                    Java.Lang.Thread.Sleep(50);                                     // memo: Raspi側で処理が間に合わないためwaitを入れる
                    if (SendMessage(sendData) == false)
                    {
                        Toast.MakeText(this, Resource.String.setting_fail, ToastLength.Short).Show();
                        return;
                    }
                }
            }

            // レスポンス確認
            switch (checkResponse())
            {
                case RESPONSE_ID.Success:
                    //Toast.MakeText(this, Resource.String.setting_accept, ToastLength.Short).Show();

                    if (settingCompleteAfterShutdown == MySetting.COMPLETE_AFTER_SHUTDOWN_KIND.KEEP)
                    {
                        Toast.MakeText(this, string.Format("{0}\n(終了後:そのまま)", GetString(Resource.String.setting_accept)), ToastLength.Short).Show();
                    }
                    else
                    {
                        Toast.MakeText(this, string.Format("{0}\n(終了後:シャットダウン)", GetString(Resource.String.setting_accept)), ToastLength.Short).Show();
                    }
                    break;

                case RESPONSE_ID.Failure:
                    Toast.MakeText(this, Resource.String.setting_fail, ToastLength.Short).Show();
                    break;

                case RESPONSE_ID.FailureForRunning:
                    Toast.MakeText(this, Resource.String.setting_fail_running, ToastLength.Short).Show();
                    break;

                case RESPONSE_ID.FailureSettingZero:
                    Toast.MakeText(this, Resource.String.setting_zero, ToastLength.Short).Show();
                    break;
            }
        }
        #endregion //「設定」ボタン


        #region 「キャンセル」ボタン
        /// <summary>「キャンセル」ボタン</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            byte[] sendData = new byte[2];

            BitConverter.GetBytes(CMD_ID_CANCEL).CopyTo(sendData, 0);   // 0～1

            if (SendMessage(sendData) == false)
            {
                Toast.MakeText (this, Resource.String.cancel_fail, ToastLength.Short).Show ();
                return;
            }

            switch (checkResponse())
            {
                case RESPONSE_ID.Success:
                    Toast.MakeText (this, Resource.String.cancel_accept, ToastLength.Short).Show ();
                    break;

                case RESPONSE_ID.Failure:
                case RESPONSE_ID.FailureForRunning:
                    Toast.MakeText (this, Resource.String.cancel_fail, ToastLength.Short).Show ();
                    break;
            }
        }
        #endregion //「キャンセル」ボタン


        #region 「実行」ボタン
        /// <summary>「実行」ボタン</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonAccept_Click(object sender, EventArgs e)
        {
            byte[] sendData = new byte[2];

            BitConverter.GetBytes(CMD_ID_ACCEPT).CopyTo(sendData, 0);   // 0～1

            if (SendMessage(sendData) == false)
            {
                Toast.MakeText (this, Resource.String.run_fail, ToastLength.Short).Show();
                return;
            }

            switch (checkResponse())
            {
                case RESPONSE_ID.Success:
                    Toast.MakeText (this, Resource.String.run_accept, ToastLength.Short).Show();
                    break;

                case RESPONSE_ID.Failure:
                    Toast.MakeText (this, Resource.String.run_fail, ToastLength.Short).Show();
                    break;

                case RESPONSE_ID.FailureForRunning:
                    Toast.MakeText (this, Resource.String.run_fail_running, ToastLength.Short).Show();
                    break;
            }
        }
        #endregion //「実行」ボタン


        #region 「リモコンコマンド取得」ボタン
        /// <summary>「リモコンコマンド取得」ボタン</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void getRemoconCommand()
        {
            byte[] sendData = new byte[2];

            BitConverter.GetBytes(CMD_ID_GET_CMD_NAME).CopyTo(sendData, 0);   // 0～1

            if (SendMessage(sendData) == false)
            {
                Toast.MakeText (this, Resource.String.get_cmd_name_fail, ToastLength.Short).Show();
                return;
            }


            byte[] buf;
            if (ReceiveMessage(out buf, 1) == false)
            {
                Toast.MakeText(this, Resource.String.get_cmd_name_fail, ToastLength.Short).Show();
                return;
            }

            byte groupCount = buf[0];
            remoconItem = new MySetTime.RemoconItem[groupCount];
            for(int i=0; i<groupCount; i++)
            {
                
                if(ReceiveMessage(out buf, CMD_LENGTH) == false)
                {
                    Toast.MakeText(this, Resource.String.get_cmd_name_fail, ToastLength.Short).Show();
                    return;
                }

                remoconItem[i].GroupName = System.Text.Encoding.ASCII.GetString(buf);

                if(ReceiveMessage(out buf, 1) == false)
                {
                    Toast.MakeText(this, Resource.String.get_cmd_name_fail, ToastLength.Short).Show();
                    return;
                }

                byte cmdCount = buf[0];
                remoconItem[i].CmdName = new string[cmdCount];
                for (int j = 0; j < cmdCount; j++)
                {
                    if(ReceiveMessage(out buf, CMD_LENGTH) == false)
                    {
                        Toast.MakeText(this, Resource.String.get_cmd_name_fail, ToastLength.Short).Show();
                        return;
                    }

                    remoconItem[i].CmdName[j] = System.Text.Encoding.ASCII.GetString(buf);
                }
            }

            saveRemoconGroup();

            switch (checkResponse())
            {
                case RESPONSE_ID.Success:
                    Toast.MakeText (this, Resource.String.get_cmd_name_accept, ToastLength.Short).Show();
                    break;

                case RESPONSE_ID.Failure:
                case RESPONSE_ID.FailureForRunning:
                    Toast.MakeText (this, Resource.String.get_cmd_name_fail, ToastLength.Short).Show();
                    break;
            }
        }
        #endregion //「リモコンコマンド取得」ボタン


        #region レスポンスの結果を判定
        /// <summary>レスポンスの結果を判定</summary>
        /// <returns><c>true</c>, if response was checked, <c>false</c> otherwise.</returns>
        private RESPONSE_ID checkResponse()
        {
            byte[] send;
            if(ReceiveMessage(out send, 3))
            {
                ushort cmdId = (ushort)((send[1] << 8) + send[0]);

                if (cmdId == CMD_ID_RESPONSE)
                {
                    return (RESPONSE_ID)send[2];
                }
            }

            return RESPONSE_ID.Failure;
        }
        #endregion //レスポンスの結果を判定


        #region OnStart
        /// <summary>OnStartイベント</summary>
		protected override void OnStart ()
		{
			base.OnStart ();
			
			if (Debug) Log.Error (TAG, "++ ON START ++");
#if !DEBUG                          // デバッグ時は無効にする			
            // Bluetoothアダプタの有効確認
			// If BT is not on, request that it be enabled.
			// setupChat() will then be called during onActivityResult
            if (!bluetoothAdapter.IsEnabled)
            {
                // Otherwise, setup the chat session
                // Bluetoothを有効にする要求(ダイアログ)を表示
                Intent enableIntent = new Intent(BluetoothAdapter.ActionRequestEnable);
                StartActivityForResult(enableIntent, REQUEST_ENABLE_BT);
            }
            else
            {
                if (btComService == null)
                {
                    SetupChat();
                }
            }
#endif
        }
        #endregion //OnStart


        #region OnResumeイベント
        /// <summary>OnResumeイベント</summary>
        protected override void OnResume ()
		{
			base.OnResume ();
			
			// Performing this check in onResume() covers the case in which BT was
			// not enabled during onStart(), so we were paused to enable it...
			// onResume() will be called when ACTION_REQUEST_ENABLE activity returns.
            if (btComService != null)
            {
                // Only if the state is STATE_NONE, do we know that we haven't started already
                if (btComService.GetState() == BluetoothService.STATE_NONE)
                {
                    // Start the Bluetooth chat services
                    btComService.Start();
                }
            }
		}
        #endregion //OnResumeイベント


        #region Bluetoothサービスの設定など
        /// <summary>Bluetoothサービスの設定など</summary>
		private void SetupChat ()
		{
			Log.Debug (TAG, "SetupChat()");
	
			// Initialize the array adapter for the conversation thread
//			conversationArrayAdapter = new ArrayAdapter<string> (this, Resource.Layout.message);
//			conversationView = FindViewById<ListView> (Resource.Id.@in);
//			conversationView.Adapter = conversationArrayAdapter;
	
			// Initialize the compose field with a listener for the return key
			//outEditText = FindViewById<EditText> (Resource.Id.edit_text_out);
			// The action listener for the EditText widget, to listen for the return key
//			outEditText.EditorAction += delegate(object sender, TextView.EditorActionEventArgs e) {
//				// If the action is a key-up event on the return key, send the message
//				if (e.ActionId == ImeAction.ImeNull && e.Event.Action == KeyEventActions.Up)
//                {
//					var message = new Java.Lang.String (((TextView) sender).Text);
//					SendMessage (message);
//                }
//			};
			
			// Initialize the send button with a listener that for click events
//			sendButton = FindViewById<Button> (Resource.Id.button_send);
//			sendButton.Click += delegate(object sender, EventArgs e) {
//				// Send a message using content of the edit text widget
//				var view = FindViewById<TextView> (Resource.Id.edit_text_out);
//				var message = new Java.Lang.String (view.Text);
//				SendMessage (message);
//			};
			
			// Initialize the BluetoothChatService to perform bluetooth connections
            btComService = new BluetoothService (this, new MyHandler (this));
			
			// Initialize the buffer for outgoing messages
			//outStringBuffer = new StringBuffer ("");


            ConnectTarget();
		}
        #endregion //Bluetoothサービスの設定など


        #region 設定保存処理
        /// <summary>設定保存処理</summary>
        private void saveSetting()
        {
            var pref = GetPreferences(FileCreationMode.Private);
            var e = pref.Edit();

            e.PutInt(SETTING_COMPLETE_AFTER_SHUTDOWN_KIND, (byte)settingCompleteAfterShutdown);

            e.PutInt(SETTING_COUNT, settingList.Count);

            for(int i=0; i<settingList.Count; i++)
            {
                e.PutBoolean(string.Format("{0}{1:D}", SETTING_ENABLE, i), settingList[i].IsEnable);
                if(settingList[i].TimerKind == MySetTime.TimerKinds.CountDown)
                {
                    e.PutBoolean(string.Format("{0}{1:D}", SETTING_TIMER_KIND, i), true);
                }
                else
                {
                    e.PutBoolean(string.Format("{0}{1:D}", SETTING_TIMER_KIND, i), false);
                }

                e.PutInt(string.Format("{0}{1:D}", SETTING_COUNT_DOWN_HOUR, i),   settingList[i].CountDownTimerHour);
                e.PutInt(string.Format("{0}{1:D}", SETTING_COUNT_DOWN_MINUTE, i), settingList[i].CountDownTimerMinute);

                e.PutInt(string.Format("{0}{1:D}", SETTING_TIME_HOUR, i),         settingList[i].TimeHour);
                e.PutInt(string.Format("{0}{1:D}", SETTING_TIME_MINUTE, i),       settingList[i].TimeMinute);

                e.PutString(string.Format("{0}{1:D}", SETTING_REMOCON_GROUP, i),  settingList[i].RemoconGroup);
                e.PutString(string.Format("{0}{1:D}", SETTING_REMOCON_CMD, i),    settingList[i].RemoconCmd);
            }

            e.Commit();
        }
        #endregion //設定保存処理


        #region リモコングループ/コマンド情報の保存処理
        /// <summary>リモコングループ/コマンド情報の保存処理</summary>
        private void saveRemoconGroup()
        {
            var pref = GetSharedPreferences(REMOCON_FILENAME, FileCreationMode.Private);
            var e = pref.Edit();

            e.PutInt(REMOCON_GROUP_COUNT, remoconItem.Length);

            for(int i=0; i<remoconItem.Length; i++)
            {
                e.PutString(string.Format("{0}{1:D}", REMOCON_GROUP_NAME, i), remoconItem[i].GroupName);

                e.PutInt(string.Format("{0}{1:D}", REMOCON_COMMAND_COUNT, i), remoconItem[i].CmdName.Length);

                for(int j=0; j<remoconItem[i].CmdName.Length; j++)
                {
                    e.PutString(string.Format("{0}{1:D}-{2:D}", REMOCON_COMMAND_NAME, i, j), remoconItem[i].CmdName[j]);
                }
            }

            e.Commit();
        }
        #endregion //リモコングループ/コマンド情報の保存処理



        #region OnPauseイベント
        /// <summary>
        /// Called as part of the activity lifecycle when an activity is going into
        ///  the background, but has not (yet) been killed.
        /// </summary>
		protected override void OnPause ()
		{
			base.OnPause ();

			if (Debug) Log.Error (TAG, "- ON PAUSE -");

            #region 設定の保存
            saveSetting();
            #endregion 設定の保存
		}
        #endregion //OnPauseイベント

		
        #region OnStopイベント
        /// <summary>
        /// Called when you are no longer visible to the user.
        /// </summary>
		protected override void OnStop ()
		{
			base.OnStop ();

			if(Debug) Log.Error (TAG, "-- ON STOP --");
            //Toast.MakeText(this, "-- ON STOP --", ToastLength.Long).Show();
		}
        #endregion //OnStopイベント


        #region OnDestroyイベント
        /// <summary>
        /// Perform any final cleanup before an activity is destroyed.
        /// </summary>
		protected override void OnDestroy ()
		{
			base.OnDestroy ();
			
			// Stop the Bluetooth chat services
            if (btComService != null)
            {
                btComService.Stop();
            }
			
            #region 当アプリによって有効にした場合、終了時にOFFする
            // memo: OnStop()を経由しない場合もある(？)ため、ここで処理をする
            if (isBluetoothEnable)
            {
                bluetoothAdapter.Disable();
            }
            #endregion

            if (Debug) Log.Error (TAG, "--- ON DESTROY ---");
		}
        #endregion //OnDestroyイベント


        #region デバイスを検出する処理
        /// <summary>デバイスを検出する処理</summary>
		private void EnsureDiscoverable ()
		{
			if (Debug) Log.Debug (TAG, "ensure discoverable");
			
            if (bluetoothAdapter.ScanMode != ScanMode.ConnectableDiscoverable)
            {
                Intent discoverableIntent = new Intent(BluetoothAdapter.ActionRequestDiscoverable);
                discoverableIntent.PutExtra(BluetoothAdapter.ExtraDiscoverableDuration, 300);
                StartActivity(discoverableIntent);
            }
		}
        #endregion //デバイスを検出する処理


        #region Bluetoothでデータ送信
		/// <summary>
		/// Sends a message.
		/// </summary>
		/// <param name='message'>
		/// A string of text to send.
		/// </param>
		private bool SendMessage (Java.Lang.String message)
		{
			// Check that we're actually connected before trying anything
            if (btComService.GetState () != BluetoothService.STATE_CONNECTED)
            {
				Toast.MakeText (this, Resource.String.not_connected, ToastLength.Short).Show ();
				return false;
			}
	
            try
            {
    			// Check that there's actually something to send
    			if (message.Length () > 0)
                {
    				// Get the message bytes and tell the BluetoothChatService to write
    				byte[] send = message.GetBytes ();
                    btComService.Write (send);
    			}
            }
            catch (System.Exception e)
            {
                ExceptionMessage(e);

                return false;
            }

            return true;
		}


        /// <summary>
        /// Sends a message.
        /// </summary>
        /// <param name='send'>
        /// A binary to send.
        /// </param>
        private bool SendMessage (byte[] send)
        {
            // Check that we're actually connected before trying anything
            if (btComService.GetState () != BluetoothService.STATE_CONNECTED)
            {
                Toast.MakeText (this, Resource.String.not_connected, ToastLength.Short).Show ();
                return false;
            }

            try
            {
                // Check that there's actually something to send
                if ((send != null) & (send.Length > 0))
                {
                    btComService.Write (send);
                }
            }
            catch (System.Exception e)
            {
                ExceptionMessage(e);

                return false;
            }

            return true;
        }
        #endregion //Bluetoothでデータ送信


        #region Bluetoothでデータ受信
        /// <summary>
        /// Receives the message.
        /// </summary>
        /// <param name="send">Send.</param>
        /// <param name="length">Length.</param>
        private bool ReceiveMessage (out byte[] send, int length)
        {
            // Check that we're actually connected before trying anything
            if (btComService.GetState () != BluetoothService.STATE_CONNECTED)
            {
                Toast.MakeText (this, Resource.String.not_connected, ToastLength.Short).Show ();
                send = new byte[0];
                return false;
            }

            try
            {
                //if (length > 0)
                {
                    btComService.Read (out send, length);
                }
            }
            catch (System.Exception e)
            {
                ExceptionMessage(e);

                send = new byte[0];
                return false;
            }

            return true;
        }
        #endregion //Bluetoothでデータ受信


        #region カウントダウンタイマー本体との接続を行う
        /// <summary>
        /// カウントダウンタイマー本体との接続を行う
        /// </summary>
        private void ConnectTarget ()
        {
            // Get the local Bluetooth adapter
            BluetoothAdapter btAdapter = BluetoothAdapter.DefaultAdapter;

            // Get a set of currently paired devices
            var pairedDevices = btAdapter.BondedDevices;

            // If there are paired devices, add each one to the ArrayAdapter
            if (pairedDevices.Count > 0)
            {
                foreach (var device in pairedDevices)
                {
                    if (device.Name.Contains(CDT_NAME))
                    {
                        btComService.Connect(device);
                    }
                }
            }
            else
            {
                Toast.MakeText(this, Resource.String.none_paired, ToastLength.Long).Show();
            }
        }
        #endregion //カウントダウンタイマー本体との接続を行う


        #region 例外エラー時の後処理
        /// <summary>例外エラー時の後処理</summary>
        /// <param name="e">E.</param>
        private void ExceptionMessage(System.Exception e)
        {
            string msg = string.Format("例外エラー!\n{0}\n--------------------\n{1}", e.Message, e.StackTrace);

            Toast.MakeText (this, msg , ToastLength.Long).Show ();
            ClipboardManager cm = (ClipboardManager)GetSystemService(Context.ClipboardService);
            cm.Text = msg;
        }
        #endregion //例外エラー時の後処理




        #region MyHandlerクラス
		// The Handler that gets information back from the BluetoothChatService
		private class MyHandler : Handler
		{
            MainActivity bluetoothCom;

            #region コンストラクタ
            /// <summary>
            /// Initializes a new instance of the <see cref="BluetoothChat.BluetoothChat+MyHandler"/> class.
            /// </summary>
            /// <param name="chat">Chat.</param>
            public MyHandler (MainActivity chat)
			{
				bluetoothCom = chat;
			}
            #endregion //コンストラクタ


            #region HandleMessage
			public override void HandleMessage (Message msg)
			{
				switch (msg.What)
                {
                    case BluetoothService.MESSAGE_STATE_CHANGE:
                        if (Debug) Log.Info(TAG, "MESSAGE_STATE_CHANGE: " + msg.Arg1);

                        switch (msg.Arg1)
                        {
                            case BluetoothService.STATE_CONNECTED:
                                bluetoothCom.textViewStatus.SetText(Resource.String.title_connected_to);
                                bluetoothCom.textViewStatus.Append(string.Format("({0})", bluetoothCom.connectedDeviceName));
                                break;

                            case BluetoothService.STATE_CONNECTING:
                                bluetoothCom.textViewStatus.SetText(Resource.String.title_connecting);
                                break;

                            case BluetoothService.STATE_LISTEN:
                            case BluetoothService.STATE_NONE:
                                bluetoothCom.textViewStatus.SetText(Resource.String.title_not_connected);
                                break;
                        }
					    break;

                    case BluetoothService.MESSAGE_WRITE:
    					// construct a string from the buffer
    					break;

                    case BluetoothService.MESSAGE_READ:
                        byte[] readBuf = (byte[])msg.Obj;
    					// construct a string from the valid bytes in the buffer
                        var readMessage = new Java.Lang.String(readBuf, 0, msg.Arg1);
    					break;

                    case BluetoothService.MESSAGE_DEVICE_NAME:
    					// save the connected device's name
                        bluetoothCom.connectedDeviceName = msg.Data.GetString(DEVICE_NAME);
                        Toast.MakeText (Application.Context, bluetoothCom.GetString(Resource.String.connected) + bluetoothCom.connectedDeviceName, ToastLength.Short).Show ();
    					break;

                    case BluetoothService.MESSAGE_TOAST:
    					Toast.MakeText (Application.Context, msg.Data.GetString (TOAST), ToastLength.Short).Show ();
    					break;
				}
			}
            #endregion //HandleMessage
		}
        #endregion //MyHandlerクラス


        #region Bluetooth 有効化の結果を受け取る
        /// <param name="requestCode">The integer request code originally supplied to
        ///  startActivityForResult(), allowing you to identify who this
        ///  result came from.</param>
        /// <param name="resultCode">The integer result code returned by the child activity
        ///  through its setResult().</param>
        /// <param name="data">An Intent, which can return result data to the caller
        ///  (various data can be attached to Intent "extras").</param>
        /// <summary>
        /// Called when an activity you launched exits, giving you the requestCode
        ///  you started it with, the resultCode it returned, and any additional
        ///  data from it.
        /// </summary>
		protected override void OnActivityResult (int requestCode, Result resultCode, Intent data)
		{
			if (Debug) Log.Debug (TAG, "onActivityResult " + resultCode);
			
			switch(requestCode)
			{
				case REQUEST_CONNECT_DEVICE:
					// When DeviceListActivity returns with a device to connect
					if( resultCode == Result.Ok)
					{
						// Get the device MAC address
						var address = data.Extras.GetString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
						// Get the BLuetoothDevice object
						BluetoothDevice device = bluetoothAdapter.GetRemoteDevice(address);
						// Attempt to connect to the device
                        btComService.Connect(device);
					}
					break;

				case REQUEST_ENABLE_BT:
					// When the request to enable Bluetooth returns
					if(resultCode == Result.Ok)
					{
                        isBluetoothEnable = true;

						// Bluetooth is now enabled, so set up a chat session
						SetupChat();
					}
					else
					{
						// User did not enable Bluetooth or an error occured
						Log.Debug(TAG, "BT not enabled");
						Toast.MakeText(this, Resource.String.bt_not_enabled_leaving, ToastLength.Short).Show();
						Finish();
					}
					break;
			}
		}
        #endregion //Bluetooth 有効化の結果を受け取る
		

        #region OnCreateOptionsMenuイベント
		public override bool OnCreateOptionsMenu (IMenu menu)
		{
			var inflater = MenuInflater;
			inflater.Inflate(Resource.Menu.option_menu, menu);
			return true;
		}
        #endregion //OnCreateOptionsMenuイベント


        #region OnOptionsItemSelected
		public override bool OnOptionsItemSelected (IMenuItem item)
		{
			switch (item.ItemId) 
			{
				case Resource.Id.scan:
					// Launch the DeviceListActivity to see devices and do scan
					var serverIntent = new Intent(this, typeof(DeviceListActivity));
					StartActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
					return true;

				case Resource.Id.discoverable:
					// Ensure this device is discoverable by others
					EnsureDiscoverable();
					return true;

                case Resource.Id.setting:               // 設定
                    var ms = new MySetting(this);                                   // 設定画面の構築
                    ms.CompleteAfterShutdown = settingCompleteAfterShutdown;

                    AlertDialog.Builder alertDlg1 = new AlertDialog.Builder(this);    // アラートダイアログ を生成
                    alertDlg1.SetTitle("設定");
                    alertDlg1.SetView(ms);
                    alertDlg1.SetPositiveButton("OK", delegate
                        {
                            // OK ボタンクリック処理
                            settingCompleteAfterShutdown = ms.CompleteAfterShutdown;
                        });

                    alertDlg1.SetNegativeButton("キャンセル", delegate
                        {
                            // Cancel ボタンクリック処理
                        });

                    alertDlg1.Create().Show();        // 表示
                    return true;

                case Resource.Id.get_remocon_command:   // リモコンコマンドを取得する
                    AlertDialog.Builder alertDlg2 = new AlertDialog.Builder(this);   // アラートダイアログ を生成
                    alertDlg2.SetTitle("確認");
                    alertDlg2.SetMessage("現在の情報を破棄して、新たにリモコンコマンドを取得します。\n宜しいですか？");
                    alertDlg2.SetPositiveButton("OK", delegate
                        {
                            // OK ボタンクリック処理
                            getRemoconCommand();
                        });

                    alertDlg2.SetNegativeButton("キャンセル", delegate
                        {
                            // Cancel ボタンクリック処理
                        });

                    alertDlg2.Create().Show();       // 表示
                    return true;
			}
			return false;
		}
        #endregion //OnOptionsItemSelected
	}
}


