﻿using System;
using Android.Util;
using Android.Content;
using Android.Views;
using Android.Widget;






/// <summary>
/// 時間設定用GUI部品
/// </summary>
namespace MyClass
{
	/// <summary>
    /// My time picker.
    /// </summary>
    public class MySetTime : LinearLayout
    {
        #region 定数
        private const string TAG = "MySetTimeClass";
        #endregion //定数



		#region 列挙体
		#region タイマー種別
		/// <summary>タイマー種別</summary>
		public enum TimerKinds
		{
			/// <summary>カウントダウンタイマー</summary>
			CountDown = 0,
			/// <summary>時刻指定</summary>
			Time,
		}
        #endregion //タイマー種別
        #endregion //列挙体



        #region 構造体
        #region 「カウントダウンタイマー」の実行に必要な情報
        /// <summary>「カウントダウンタイマー」の実行に必要な情報</summary>
        public struct TimerExecInfo
        {
            public bool IsEnable;
            public TimerKinds TimerKind;
            public int CountDownTimerHour;
            public int CountDownTimerMinute;
            public int TimeHour;
            public int TimeMinute;
            public string RemoconGroup;
            public string RemoconCmd;

            #region 変数を初期化します
            /// <summary>変数を初期化します</summary>
            public void Initialize()
            {
                IsEnable                = false;
                TimerKind               = TimerKinds.CountDown;
                CountDownTimerHour      = 0;
                CountDownTimerMinute    = 0;
                TimeHour                = 0;
                TimeMinute              = 0;
                RemoconGroup            = "";
                RemoconCmd              = "";
            }
            #endregion //変数を初期化します
        }
        #endregion //「カウントダウンタイマー」の実行に必要な情報


        #region リモコングループ/コマンド名管理用構造体
        /// <summary>リモコングループ/コマンド名管理用構造体</summary>
        public struct RemoconItem
        {
            /// <summary>リモコングループ</summary>
            public string   GroupName;
            /// <summary>リモコンコマンド</summary>
            public string[] CmdName;
        }
        #endregion //リモコングループ/コマンド名管理用構造体
        #endregion //構造体



        #region フィールド
        /// <summary></summary>
        private LinearLayout linearLayoutBack;
        /// <summary></summary>
        private ScrollView scrollViewBack;

        /// <summary></summary>
        private LinearLayout linearLayoutTimerKind;
        /// <summary></summary>
        private TextView textViewTimerKind;
        /// <summary></summary>
        private RadioGroup radioGroup;
        /// <summary></summary>
        private RadioButton radioButtonCountDownTimer;
        /// <summary></summary>
        private RadioButton radioButtonTime;
        /// <summary></summary>
        private MyTimePicker99h myTimePickerCountDown;
        /// <summary></summary>
        private MyTimePicker myTimePickerTime;

        /// <summary></summary>
        private LinearLayout linearLayoutRemocon;
        /// <summary></summary>
        private TextView textViewRemoconGroup;
        /// <summary></summary>
        private TextView textViewRemoconCmd;

        /// <summary></summary>
        private ArrayAdapter<string> remoconGroupAdapter;
        /// <summary></summary>
        private Spinner spinnerRemoconGroup;

        /// <summary></summary>
        private ArrayAdapter<string> remoconCmdAdapter;
        /// <summary></summary>
        private Spinner spinnerRemoconCmd;


        /// <summary></summary>
        private TimerExecInfo timerInfo = new TimerExecInfo();

        /// <summary></summary>
        private RemoconItem[] remoconItem;

        #endregion //フィールド




        #region プロパティ
        #region 「カウントダウンタイマー」の実行に必要な情報
        /// <summary>「カウントダウンタイマー」の実行に必要な情報</summary>
        public TimerExecInfo TimerInfo
        {
            get
            {
                /// <summary>カウントダウンタイマー:時</summary>
                timerInfo.CountDownTimerHour    = myTimePickerCountDown.CurrentHour;
                /// <summary>カウントダウンタイマー:分</summary>
                timerInfo.CountDownTimerMinute  = myTimePickerCountDown.CurrentMinute;
                /// <summary>時刻指定:時</summary>
                timerInfo.TimeHour              = myTimePickerTime.CurrentHour;
                /// <summary>時刻指定:分</summary>
                timerInfo.TimeMinute            = myTimePickerTime.CurrentMinute;

                timerInfo.RemoconGroup = spinnerRemoconGroup.SelectedItem.ToString();
                timerInfo.RemoconCmd = spinnerRemoconCmd.SelectedItem.ToString();

                return timerInfo;
            }
            set
            {
                timerInfo = value;

                myTimePickerCountDown.CurrentHour   = timerInfo.CountDownTimerHour;
                myTimePickerCountDown.CurrentMinute = timerInfo.CountDownTimerMinute;

                myTimePickerTime.CurrentHour        = timerInfo.TimeHour;
                myTimePickerTime.CurrentMinute      = timerInfo.TimeMinute;

                if (timerInfo.TimerKind == TimerKinds.CountDown)
                {
                    radioButtonCountDownTimer.Checked   = true;
                    radioButtonTime.Checked             = false;
                }
                else
                {
                    radioButtonTime.Checked             = true;
                    radioButtonCountDownTimer.Checked   = false;
                }

                setRemoconGroup();
                setRemoconCommand();

                radioButtonCheckedChange();
            }
        }
        #endregion //「カウントダウンタイマー」の実行に必要な情報


        #region RemoconGroup
        /// <summary>RemoconGroup</summary>
        /// <value></value>
        public RemoconItem[] RemoconItems
        {
            set
            {
                remoconItem = value;
            }
        }
        #endregion //RemoconGroup
        #endregion //プロパティ






    
        #region コンストラクタ
        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        public MySetTime(Context context) : base(context)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        public MySetTime(Context context, IAttributeSet attrs) : base(context, attrs)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        /// <param name="defStyle">Def style.</param>
        public MySetTime(Context context, IAttributeSet attrs, int defStyle) : base(context, attrs, defStyle)
        {
            InitializeComponent(context);
        }
        #endregion //コンストラクタ



        #region コンポーネントの初期化
        /// <summary>コンポーネントの初期化</summary>
        private void InitializeComponent(Context context)
        {
            // .axmlではなくコードで配置する
			this.Orientation = Orientation.Vertical;

            // マージン(外側に対する余白)
            LayoutParams lp = new LayoutParams(LayoutParams.MatchParent, LayoutParams.WrapContent);
            lp.SetMargins(5, 5, 5, 5);


            // 「バック」グループ
            linearLayoutBack= new LinearLayout(context);
            linearLayoutBack.Orientation = Orientation.Vertical;

            // 「タイマー種類」グループ
            linearLayoutTimerKind = new LinearLayout(context);
            linearLayoutTimerKind.Orientation = Orientation.Vertical;
            linearLayoutTimerKind.SetBackgroundDrawable(Resources.GetDrawable(IrTimer.Resource.Drawable.custom_frame));
            linearLayoutTimerKind.LayoutParameters = lp;

            textViewTimerKind = new TextView(context);
            textViewTimerKind.Text = "タイマー種類";

            radioButtonCountDownTimer = new RadioButton(context);
            radioButtonCountDownTimer.Text = "カウントダウン";

            radioButtonTime = new RadioButton(context);
            radioButtonTime.Text = "時刻指定";

            radioGroup = new RadioGroup(context);
            radioGroup.AddView(radioButtonCountDownTimer);
            radioGroup.AddView(radioButtonTime);

            linearLayoutTimerKind.AddView(textViewTimerKind);
            linearLayoutTimerKind.AddView(radioGroup);


            // 「カウントダウン」グループ
			myTimePickerCountDown = new MyTimePicker99h (context);
			myTimePickerCountDown.SetBackgroundDrawable(Resources.GetDrawable(IrTimer.Resource.Drawable.custom_frame));
            myTimePickerCountDown.LayoutParameters = lp;


            // 「時刻指定」グループ
			myTimePickerTime = new MyTimePicker (context);
			myTimePickerTime.SetBackgroundDrawable(Resources.GetDrawable(IrTimer.Resource.Drawable.custom_frame));
			myTimePickerTime.Visibility = ViewStates.Gone;
            myTimePickerTime.LayoutParameters = lp;


            // 「リモコン」グループ
            linearLayoutRemocon = new LinearLayout(context);
            linearLayoutRemocon.Orientation = Orientation.Vertical;
            linearLayoutRemocon.SetBackgroundDrawable(Resources.GetDrawable(IrTimer.Resource.Drawable.custom_frame));
            linearLayoutRemocon.LayoutParameters = lp;

            textViewRemoconGroup = new TextView(context);
            textViewRemoconGroup.Text = "グループ：";

            textViewRemoconCmd = new TextView(context);
            textViewRemoconCmd.Text = "コマンド：";

            spinnerRemoconGroup = new Spinner(context);
            remoconGroupAdapter = new ArrayAdapter<string>(context, Android.Resource.Layout.SimpleSpinnerItem);
            remoconGroupAdapter.SetDropDownViewResource(Android.Resource.Layout.SimpleSpinnerDropDownItem);
            spinnerRemoconGroup.Adapter = remoconGroupAdapter;
            spinnerRemoconGroup.ItemSelected += new EventHandler<AdapterView.ItemSelectedEventArgs>(spinnerRemoconGroup_ItemSelected);

            spinnerRemoconCmd = new Spinner(context);
            remoconCmdAdapter = new ArrayAdapter<string>(context, Android.Resource.Layout.SimpleSpinnerItem);
            remoconCmdAdapter.SetDropDownViewResource(Android.Resource.Layout.SimpleSpinnerDropDownItem);
            spinnerRemoconCmd.Adapter = remoconCmdAdapter;
            spinnerRemoconCmd.ItemSelected += delegate
                {
                    // todo:
                };
            
            linearLayoutRemocon.AddView(textViewRemoconGroup);
            linearLayoutRemocon.AddView(spinnerRemoconGroup);
            linearLayoutRemocon.AddView(textViewRemoconCmd);
            linearLayoutRemocon.AddView(spinnerRemoconCmd);



            // 配置
            linearLayoutBack.AddView(linearLayoutTimerKind);
            linearLayoutBack.AddView(myTimePickerCountDown);
            linearLayoutBack.AddView(myTimePickerTime);
            linearLayoutBack.AddView(linearLayoutRemocon);


            scrollViewBack = new ScrollView(context);
            scrollViewBack.AddView(linearLayoutBack);
            this.AddView(scrollViewBack);


            // イベント
            // コードで配置すると、なぜかチェックのトグル処理が働かないため、自前で処理する
            // 実機では不要だが、エミュレータでうまく動作しないため記述しておく。
            radioButtonCountDownTimer.Click += new EventHandler(radioButtonCountDownTimer_Click);
            radioButtonTime.Click           += new EventHandler(radioButtonTime_Click);

            radioGroup.CheckedChange += new EventHandler<RadioGroup.CheckedChangeEventArgs>(radioGroup_CheckedChange);
            
            return;
        }
        #endregion //コンポーネントの初期化    
    

		#region タイマー種類に応じた処理
		/// <summary>タイマー種類に応じた処理</summary>
		/// <param name="sender">Sender.</param>
		/// <param name="e">E.</param>
		private void radioGroup_CheckedChange(object sender, RadioGroup.CheckedChangeEventArgs e)
		{
            Log.WriteLine(LogPriority.Info, TAG, "radioGroup_CheckedChange");

            radioButtonCheckedChange();
		}
        #endregion //タイマー種類に応じた処理


        #region タイマー種類に応じた処理
        /// <summary>タイマー種類に応じた処理</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void radioButtonCheckedChange()
        {
            Log.WriteLine(LogPriority.Info, TAG, "radioButtonCheckedChange");

            if (radioButtonCountDownTimer.Checked)
            {
                timerInfo.TimerKind = TimerKinds.CountDown;

                myTimePickerTime.Visibility         = ViewStates.Gone;
                myTimePickerCountDown.Visibility    = ViewStates.Visible;

                myTimePickerTime.RequestFocus();     // memo: フォーカスが当たってソフトキーボードは表示されてしまうため。現状これしか有効な対策がない。
            }
            else
            {
                timerInfo.TimerKind = TimerKinds.Time;

                myTimePickerCountDown.Visibility    = ViewStates.Gone;
                myTimePickerTime.Visibility         = ViewStates.Visible;
            }
        }
        #endregion //タイマー種類に応じた処理


        #region ラジオボタンのトグル処理(radioButtonCountDownTimer)
        /// <summary>ラジオボタンのトグル処理(radioButtonCountDownTimer)</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void radioButtonCountDownTimer_Click(object sender, EventArgs e)
        {
            Log.WriteLine(LogPriority.Info, TAG, "radioButtonCountDownTimer_Click");

            radioButtonTime.Checked = false;            // 他方をfalseする
        }
        #endregion //ラジオボタンのトグル処理(radioButtonCountDownTimer)


        #region ラジオボタンのトグル処理(radioButtonTime)
        /// <summary>ラジオボタンのトグル処理(radioButtonTime)</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void radioButtonTime_Click(object sender, EventArgs e)
        {
            Log.WriteLine(LogPriority.Info, TAG, "radioButtonTime_Click");

            radioButtonCountDownTimer.Checked = false;  // 他方をfalseする
        }
        #endregion //ラジオボタンのトグル処理(radioButtonTime)


        #region リモコングループを選択した時の処理
        /// <summary>リモコングループを選択した時の処理</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void spinnerRemoconGroup_ItemSelected(object sender, AdapterView.ItemSelectedEventArgs e)
        {
            Log.WriteLine(LogPriority.Info, TAG, "ItemSelected()");

            setRemoconCommand();
        }
        #endregion //リモコングループを選択した時の処理


        #region リモコングループの設定
        /// <summary>リモコングループの設定</summary>
        private void setRemoconGroup()
        {
            Log.WriteLine(LogPriority.Info, TAG, "setRemoconGroup()");

            remoconGroupAdapter.Clear();
            int pos = 0;
            for (int i = 0; i < remoconItem.Length; i++)        // memo: 一旦spinner.Adapterにセットした後でないとspinner.SetSelectionが有効にならないため、処理を分ける
            {
                remoconGroupAdapter.Add(remoconItem[i].GroupName);

                if (remoconItem[i].GroupName == timerInfo.RemoconGroup)
                {
                    pos = i;
                }
            }
            spinnerRemoconGroup.Adapter = remoconGroupAdapter;
            spinnerRemoconGroup.SetSelection(pos);
        }
        #endregion //リモコングループの設定


        #region リモコンコマンドの設定
        /// <summary>リモコンコマンドの設定</summary>
        private void setRemoconCommand()
        {
            Log.WriteLine(LogPriority.Info, TAG, "setRemoconCommand()");

            if (remoconItem.Length > 0)
            {
                remoconCmdAdapter.Clear();
                int pos = 0;
                for (int i = 0; i < remoconItem[spinnerRemoconGroup.SelectedItemPosition].CmdName.Length; i++)
                {
                    remoconCmdAdapter.Add(remoconItem[spinnerRemoconGroup.SelectedItemPosition].CmdName[i]);

                    if (remoconItem[spinnerRemoconGroup.SelectedItemPosition].CmdName[i] == timerInfo.RemoconCmd)
                    {
                        pos = i;
                    }
                }

                spinnerRemoconCmd.Adapter = remoconCmdAdapter;
                spinnerRemoconCmd.SetSelection(pos);
            }
        }
        #endregion //リモコンコマンドの設定

    }
}

