﻿using System;
using Android.Util;
using Android.Content;
using Android.Views;
using Android.Widget;






/// <summary>
/// 時間設定用GUI部品
/// </summary>
namespace MyClass
{
	/// <summary>
    /// My time picker.
    /// </summary>
    public class MySetting : LinearLayout
    {
        #region 定数
        private const string TAG = "MySettingClass";
        #endregion //定数


		#region 列挙体
        #region 設定:終了後の動作
        /// <summary>設定:終了後の動作</summary>
        public enum COMPLETE_AFTER_SHUTDOWN_KIND : byte
        {
            /// <summary>実行終了後もそのまま</summary>
            KEEP        = 0,
            /// <summary>実行が終わったらシャットダウンする</summary>
            SHUTDOWN    = 1,
        }
        #endregion //設定:終了後の動作
        #endregion //列挙体


        #region フィールド
        /// <summary>設定:終了後の動作</summary>
        private COMPLETE_AFTER_SHUTDOWN_KIND completeAfterShutdown = COMPLETE_AFTER_SHUTDOWN_KIND.KEEP;

        /// <summary></summary>
        private LinearLayout linearLayoutBack;

        /// <summary></summary>
        private LinearLayout linearLayoutCompleteAfterShutdownKindKind;
        /// <summary></summary>
        private TextView textViewCompleteAfterShutdownKindKind;
        /// <summary></summary>
        private RadioGroup radioGroup;
        /// <summary></summary>
        private RadioButton radioButtonKeep;
        /// <summary></summary>
        private RadioButton radioButtonShutdown;
        #endregion //フィールド



        #region プロパティ
        #region 設定:終了後の動作
        /// <summary>設定:終了後の動作</summary>
        public COMPLETE_AFTER_SHUTDOWN_KIND CompleteAfterShutdown
        {
            get
            {
                return completeAfterShutdown;
            }
            set
            {
                completeAfterShutdown = value;

                if (completeAfterShutdown == COMPLETE_AFTER_SHUTDOWN_KIND.KEEP)
                {
                    radioButtonKeep.Checked = true;
                    radioButtonShutdown.Checked = false;
                }
                else
                {
                    radioButtonShutdown.Checked = true;
                    radioButtonKeep.Checked = false;
                }
            }
        }
        #endregion //設定:終了後の動作



        #endregion //プロパティ


    
        #region コンストラクタ
        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        public MySetting(Context context) : base(context)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        public MySetting(Context context, IAttributeSet attrs) : base(context, attrs)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        /// <param name="defStyle">Def style.</param>
        public MySetting(Context context, IAttributeSet attrs, int defStyle) : base(context, attrs, defStyle)
        {
            InitializeComponent(context);
        }
        #endregion //コンストラクタ



        #region コンポーネントの初期化
        /// <summary>コンポーネントの初期化</summary>
        private void InitializeComponent(Context context)
        {
            // .axmlではなくコードで配置する

			this.Orientation = Orientation.Vertical;

            // マージン(外側に対する余白)
            LayoutParams lp = new LayoutParams(LayoutParams.MatchParent, LayoutParams.WrapContent);
            lp.SetMargins(5, 5, 5, 5);

            // 「バック」グループ
            linearLayoutBack= new LinearLayout(context);
            linearLayoutBack.Orientation = Orientation.Vertical;

            // 「終了後の動作」グループ
            linearLayoutCompleteAfterShutdownKindKind= new LinearLayout(context);
            linearLayoutCompleteAfterShutdownKindKind.Orientation = Orientation.Vertical;
            linearLayoutCompleteAfterShutdownKindKind.SetBackgroundDrawable(Resources.GetDrawable(IrTimer.Resource.Drawable.custom_frame));
            linearLayoutCompleteAfterShutdownKindKind.LayoutParameters = lp;

            textViewCompleteAfterShutdownKindKind = new TextView(context);
            textViewCompleteAfterShutdownKindKind.Text = "終了後の動作：";

            radioButtonKeep = new RadioButton(context);
            radioButtonKeep.Text = "そのまま";
            radioButtonShutdown= new RadioButton(context);
            radioButtonShutdown.Text = "シャットダウンする";

            radioGroup = new RadioGroup(context);
            radioGroup.AddView(radioButtonKeep);
            radioGroup.AddView(radioButtonShutdown);

            linearLayoutCompleteAfterShutdownKindKind.AddView(textViewCompleteAfterShutdownKindKind);
            linearLayoutCompleteAfterShutdownKindKind.AddView(radioGroup);


            // 配置
            linearLayoutBack.AddView(linearLayoutCompleteAfterShutdownKindKind);
            this.AddView(linearLayoutBack);

            // コードで配置すると、なぜかチェックのトグル処理が働かないため、自前で処理する
            // 実機では不要だが、エミュレータでうまく動作しないため記述しておく。
            radioButtonKeep.Click       += new EventHandler(radioButtonKeep_Click);
            radioButtonShutdown.Click   += new EventHandler(radioButtonShutdown_Click);

            radioGroup.CheckedChange += new EventHandler<RadioGroup.CheckedChangeEventArgs>(radioGroup_CheckedChange);
            
            return;
        }
        #endregion //コンポーネントの初期化    
    

		#region 終了後の動作に応じた処理
        /// <summary>終了後の動作に応じた処理</summary>
		/// <param name="sender">Sender.</param>
		/// <param name="e">E.</param>
		private void radioGroup_CheckedChange(object sender, RadioGroup.CheckedChangeEventArgs e)
		{
            Log.WriteLine(LogPriority.Info, TAG, "radioGroup_CheckedChange");

            radioButtonCheckedChange();
		}
        #endregion //終了後の動作に応じた処理


        #region 終了後の動作に応じた処理
        /// <summary>終了後の動作に応じた処理</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void radioButtonCheckedChange()
        {
            Log.WriteLine(LogPriority.Info, TAG, "radioButtonCheckedChange");

            if (radioButtonKeep.Checked)
            {
                completeAfterShutdown = COMPLETE_AFTER_SHUTDOWN_KIND.KEEP;
            }
            else
            {
                completeAfterShutdown = COMPLETE_AFTER_SHUTDOWN_KIND.SHUTDOWN;
            }
        }
        #endregion //終了後の動作に応じた処理


        #region ラジオボタンのトグル処理(radioButtonCountDownTimer)
        /// <summary>ラジオボタンのトグル処理(radioButtonCountDownTimer)</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void radioButtonKeep_Click(object sender, EventArgs e)
        {
            Log.WriteLine(LogPriority.Info, TAG, "radioButtonCountDownTimer_Click");

            radioButtonShutdown.Checked = false;            // 他方をfalseする
        }
        #endregion //ラジオボタンのトグル処理(radioButtonCountDownTimer)


        #region ラジオボタンのトグル処理(radioButtonTime)
        /// <summary>ラジオボタンのトグル処理(radioButtonTime)</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void radioButtonShutdown_Click(object sender, EventArgs e)
        {
            Log.WriteLine(LogPriority.Info, TAG, "radioButtonTime_Click");

            radioButtonKeep.Checked = false;  // 他方をfalseする
        }
        #endregion //ラジオボタンのトグル処理(radioButtonTime)


    }
}

