﻿using System;
using Android.Util;
using Android.Content;
using Android.Views;
using Android.Widget;


/// <summary>
/// TimePickerを拡張した物。
/// ・+10分/-10分ボタンを追加した物
/// </summary>
namespace MyClass
{
	/// <summary>
    /// My time picker.
    /// </summary>
    public class MyTimePicker : LinearLayout
    {


        #region フィールド
        /// <summary></summary>
        private TimePicker timePicker;
        /// <summary></summary>
        private LinearLayout linearLayoutMinuteButton;
        /// <summary></summary>
        private Button buttonAdd10;
        /// <summary></summary>
        private Button buttonSub10;

        #endregion //フィールド



        #region プロパティ
        #region 時
        /// <summary>時</summary>
        public int CurrentHour
        {
            get
            {
                return timePicker.CurrentHour.IntValue();
            }
            set
            {
                timePicker.CurrentHour = (Java.Lang.Integer)value;
            }
        }
        #endregion //時


        #region 分
        /// <summary>分</summary>
        public int CurrentMinute
        {
            get
            {
                return timePicker.CurrentMinute.IntValue();
            }
            set
            {
                timePicker.CurrentMinute = (Java.Lang.Integer)value;
            }
        }
        #endregion //分
        #endregion //プロパティ


    
        #region コンストラクタ
        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        public MyTimePicker(Context context) : base(context)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        public MyTimePicker(Context context, IAttributeSet attrs) : base(context, attrs)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        /// <param name="defStyle">Def style.</param>
        public MyTimePicker(Context context, IAttributeSet attrs, int defStyle) : base(context, attrs, defStyle)
        {
            InitializeComponent(context);
        }
        #endregion //コンストラクタ



        #region コンポーネントの初期化
        /// <summary>コンポーネントの初期化</summary>
        private void InitializeComponent(Context context)
        {
            // .axmlではなくコードで配置する

            this.Orientation = Orientation.Horizontal;
            this.SetGravity(GravityFlags.Center);

            timePicker = new TimePicker(context);

            linearLayoutMinuteButton = new LinearLayout(context);
            linearLayoutMinuteButton.Orientation = Orientation.Vertical;
            linearLayoutMinuteButton.SetGravity(GravityFlags.Center);

            buttonAdd10      = new Button(Context);
            buttonSub10      = new Button(Context);

            buttonAdd10.Text = "+10";
            buttonSub10.Text = "-10";

            this.AddView(timePicker);

            linearLayoutMinuteButton.AddView(buttonAdd10);
            linearLayoutMinuteButton.AddView(buttonSub10);
            this.AddView(linearLayoutMinuteButton);

            buttonAdd10.Click += new EventHandler(buttonAdd10_Click);
            buttonSub10.Click += new EventHandler(buttonSub10_Click);

            return;
        }
        #endregion //コンポーネントの初期化
    

        #region 「+10」ボタンを押下した時のイベント
        /// <summary>「+10」ボタンを押下した時のイベント</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonAdd10_Click(object sender, EventArgs e)
        {
            int m = timePicker.CurrentMinute.IntValue();

            m = (m / 10) * 10;  // 10の位のみ抜き出す処理
            if (m == 50)
            {
                int h = timePicker.CurrentHour.IntValue();
                if (h == 23)
                {
                    h = 0;
                }
                else
                {
                    h++;
                }

                timePicker.CurrentHour = (Java.Lang.Integer)h;
                m = 0;
            }
            else
            {
                m = m + 10;
            }

            timePicker.CurrentMinute = (Java.Lang.Integer)m;
        }
        #endregion //「+10」ボタンを押下した時のイベント


        #region 「-10」ボタンを押下した時のイベント
        /// <summary>「-10」ボタンを押下した時のイベント</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonSub10_Click(object sender, EventArgs e)
        {
            int m = timePicker.CurrentMinute.IntValue();

            if ((m % 10) > 0)       // 1の位に端数がある場合
            {
                m = (m / 10) * 10;  // 10の位のみ抜き出す処理
            }
            else if (m == 0)
            {
                int h = timePicker.CurrentHour.IntValue();
                if (h == 0)
                {
                    h = 23;
                }
                else
                {
                    h--;
                }
                timePicker.CurrentHour = (Java.Lang.Integer)h;
                m = 50;

            }
            else
            {
                m = m - 10;
            }

            timePicker.CurrentMinute = (Java.Lang.Integer)m;
        }
        #endregion //「-10」ボタンを押下した時のイベント
    
    
    }
}

