﻿using System;
using Android.Util;
using Android.Content;
using Android.Views;
using Android.Widget;


/// <summary>
/// TimePickerを拡張した物。
/// ・99時間まで対応
/// ・+10分/-10分ボタンを追加した物
/// </summary>
namespace MyClass
{
	/// <summary>
    /// My time picker.
    /// </summary>
    public class MyTimePicker99h : LinearLayout
    {
        #region フィールド
        /// <summary></summary>
        private NumberPicker numberPickerHour;
        /// <summary></summary>
        private NumberPicker numberPickerMin;
        /// <summary></summary>
        private TextView textViewColon;

        private LinearLayout linearLayoutMinuteButton;
        /// <summary></summary>
        private Button buttonAdd10;
        /// <summary></summary>
        private Button buttonSub10;

        #endregion //フィールド



        #region プロパティ
        #region 時
        /// <summary>時</summary>
        public int CurrentHour
        {
            get
            {
                return numberPickerHour.Value;
            }
            set
            {
                numberPickerHour.Value = value;
            }
        }
        #endregion //時


        #region 分
        /// <summary>分</summary>
        public int CurrentMinute
        {
            get
            {
                return numberPickerMin.Value;
            }
            set
            {
                numberPickerMin.Value = value;
            }
        }
        #endregion //分
        #endregion //プロパティ


    
        #region コンストラクタ
        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        public MyTimePicker99h(Context context) : base(context)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        public MyTimePicker99h(Context context, IAttributeSet attrs) : base(context, attrs)
        {
            InitializeComponent(context);
        }


        /// <summary>コンストラクタ</summary>
        /// <param name="context">Context.</param>
        /// <param name="attrs">Attrs.</param>
        /// <param name="defStyle">Def style.</param>
        public MyTimePicker99h(Context context, IAttributeSet attrs, int defStyle) : base(context, attrs, defStyle)
        {
            InitializeComponent(context);
        }
        #endregion //コンストラクタ



        #region コンポーネントの初期化
        /// <summary>コンポーネントの初期化</summary>
        private void InitializeComponent(Context context)
        {
            // .axmlではなくコードで配置する
            this.Orientation = Orientation.Horizontal;
            this.SetGravity(GravityFlags.Center);

            numberPickerHour = new NumberPicker(context);
            numberPickerMin  = new NumberPicker(context);
            textViewColon    = new TextView(context);


            linearLayoutMinuteButton = new LinearLayout(context);
            linearLayoutMinuteButton.Orientation = Orientation.Vertical;
            linearLayoutMinuteButton.SetGravity(GravityFlags.Center);

            buttonAdd10      = new Button(Context);
            buttonSub10      = new Button(Context);


            numberPickerHour.Focusable = false;
            numberPickerHour.FocusableInTouchMode = false;

            numberPickerHour.SetMinimumWidth(68);
            numberPickerMin.SetMinimumWidth(68);

            numberPickerHour.MinValue = 0;
            numberPickerHour.MaxValue = 99;

            numberPickerMin.MinValue = 0;
            numberPickerMin.MaxValue = 59;

            textViewColon.Text = " : ";
            buttonAdd10.Text = "+10";
            buttonSub10.Text = "-10";

            this.AddView(numberPickerHour);
            this.AddView(textViewColon);
            this.AddView(numberPickerMin);

            linearLayoutMinuteButton.AddView(buttonAdd10);
            linearLayoutMinuteButton.AddView(buttonSub10);
            this.AddView(linearLayoutMinuteButton);

            numberPickerHour.ValueChanged += new EventHandler<NumberPicker.ValueChangeEventArgs>(numberPickerHour_ValueChanged);
            numberPickerMin.ValueChanged += new EventHandler<NumberPicker.ValueChangeEventArgs>(numberPickerMin_ValueChanged);

            buttonAdd10.Click += new EventHandler(buttonAdd10_Click);
            buttonSub10.Click += new EventHandler(buttonSub10_Click);

            return;
        }
        #endregion //コンポーネントの初期化    
    

        #region 「時間」を操作した時に起きるイベント
        /// <summary>「時間」を操作した時に起きるイベント</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void numberPickerHour_ValueChanged(object sender, NumberPicker.ValueChangeEventArgs e)
        {
            // Todo: 
        }
        #endregion //「時間」を操作した時に起きるイベント


        #region 「分」を操作した時に起きるイベント
        /// <summary>「分」を操作した時に起きるイベント</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void numberPickerMin_ValueChanged(object sender, NumberPicker.ValueChangeEventArgs e)
        {
            if ((e.OldVal == 59) & (e.NewVal == 0))
            {
                numberPickerHour.Value++;
            }
            else if((e.OldVal == 0) & (e.NewVal == 59))
            {
                numberPickerHour.Value--;
            }
        }
        #endregion //「分」を操作した時に起きるイベント
    

        #region 「+10」ボタンを押下した時のイベント
        /// <summary>「+10」ボタンを押下した時のイベント</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonAdd10_Click(object sender, EventArgs e)
        {
            int m = numberPickerMin.Value;

            m = (m / 10) * 10;  // 10の位のみ抜き出す処理
            if (m == 50)
            {
                numberPickerHour.Value++;
                m = 0;
            }
            else
            {
                m = m + 10;
            }

            numberPickerMin.Value = m;
        }
        #endregion //「+10」ボタンを押下した時のイベント


        #region 「-10」ボタンを押下した時のイベント
        /// <summary>「-10」ボタンを押下した時のイベント</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">E.</param>
        private void buttonSub10_Click(object sender, EventArgs e)
        {
            int m = numberPickerMin.Value;

            if ((m % 10) > 0)       // 1の位に端数がある場合
            {
                m = (m / 10) * 10;  // 10の位のみ抜き出す処理
            }
            else if (m == 0)
            {
                numberPickerHour.Value--;
                m = 50;
            }
            else
            {
                m = m - 10;
            }

            numberPickerMin.Value = m;
        }
        #endregion //「-10」ボタンを押下した時のイベント
    
    
    }
}

